module Collision where

    import Window
    import Pacman
    import Ghost
    import Coins

    -- | Retorna se a bola colidiu com o topo
    topCollision :: Position -> Bool
    topCollision (x, y)   = y + 2 * pacmanRadius > 215

    -- | Retorna se a bola colidiu com a parte de baixo do mapa
    downCollision :: Position -> Bool
    downCollision (x, y)   = y - 2 * pacmanRadius < -215

    -- | Retorna se a bola colidiu com a parte esquerda
    leftCollision :: Position -> Bool
    leftCollision (x, y)  = x - 2 * pacmanRadius <= -215

    -- | Retorna se a bola colidiu com a parte direita
    rightCollision :: Position -> Bool
    rightCollision (x, y) = x + 2 * pacmanRadius >=  215

    -- | Colisão: bloco horizontal - width 90 height 10 
    wallCollision :: Position -> Bool
    wallCollision (x,y) = (y - pacmanRadius <= 50 && y + pacmanRadius >= 40) && 
                          (x - pacmanRadius <= 45 && x + pacmanRadius >= (-45))

    -- | Colisão: bloco horizontal - width 90 height 10
    wallCollision1 :: Position -> Bool
    wallCollision1 (x,y) = (y - pacmanRadius <= (-40) && y + pacmanRadius >= (-50)) && 
                           (x - pacmanRadius <= 45 && x + pacmanRadius >= (-45))

    -- | Colisão: bloco vertical - width 15 height 90
    wallCollision2 :: Position -> Bool
    wallCollision2 (x,y) = (y - pacmanRadius <= 135 && y + pacmanRadius >= 45 && 
                            (x - pacmanRadius <= 145 && x + pacmanRadius >= 130))

    -- | Colisão: bloco vertical - width 15 height 90
    wallCollision3 :: Position -> Bool
    wallCollision3 (x,y) = (y - pacmanRadius <= 135 && y + pacmanRadius >= 45 && 
                            (x - pacmanRadius <= (-130) && x + pacmanRadius >= (-145)))

    -- | Colisão: bloco vertical - width 15 height 90
    wallCollision4 :: Position -> Bool
    wallCollision4 (x,y) = (y - pacmanRadius <= (-45) && y + pacmanRadius >= (-135) && 
                            (x - pacmanRadius <= 145 && x + pacmanRadius >= 130))

    -- | Colisão: bloco vertical - width 15 height 90
    wallCollision5 :: Position -> Bool
    wallCollision5 (x,y) = (y - pacmanRadius <= (-45) && y + pacmanRadius >= (-135) && 
                            (x - pacmanRadius <= (-130) && x + pacmanRadius >= (-145)))

    -- | Colisão: bloco horizontal - width 45 height 18
    wallCollision6 :: Position -> Bool
    wallCollision6 (x,y) = (y - pacmanRadius <= (145) && y + pacmanRadius >= (127) && 
                           (x - pacmanRadius <= (135) && x + pacmanRadius >= (90)))

    -- | Colisão: bloco horizontal - width 45 height 18
    wallCollision7 :: Position -> Bool
    wallCollision7 (x,y) = (y - pacmanRadius <= 145 && y + pacmanRadius >= 127 && 
                            (x - pacmanRadius <= (-90) && x + pacmanRadius >= (-135)))

    -- | Colisão: bloco horizontal - width 45 height 18
    wallCollision8 :: Position -> Bool
    wallCollision8 (x,y) = (y - pacmanRadius <= (-125) && y + pacmanRadius >= (-145) && 
                            (x - pacmanRadius <= (135) && x + pacmanRadius >= (90)))

    -- | Colisão: bloco horizontal - width 45 height 18
    wallCollision9 :: Position -> Bool
    wallCollision9 (x,y) = (y - pacmanRadius <= (-125) && y + pacmanRadius >= (-145) && 
                            (x - pacmanRadius <= (-90) && x + pacmanRadius >= (-135)))

    -- | Colisão: bloco vertical - width 12 height 90
    wallCollision10 :: Position -> Bool
    wallCollision10 (x,y) = (y - pacmanRadius <= (225) && y + pacmanRadius >= (135) && 
                            (x - pacmanRadius <= (5) && x + pacmanRadius >= (-7)))

    -- | Colisão: bloco vertical - width 12 height 90
    wallCollision11 :: Position -> Bool
    wallCollision11 (x,y) = (y - pacmanRadius <= (-135) && y + pacmanRadius >= (-225) && 
                            (x - pacmanRadius <= (5) && x + pacmanRadius >= (-7)))

    -- | Colisão do fantasma com a parede esquerda
    leftCollisionGhost :: Position -> Bool
    leftCollisionGhost (x, y)  = x - 2 * ghostRadius <= -208

    -- | Colisão do fantasma com a parede direita
    rightCollisionGhost :: Position -> Bool
    rightCollisionGhost (x, y) = x + 2 * ghostRadius >=  212

    -- | Detecta colisão da bola com as bordas e atualiza velocidade do PACMAN.
    wallBounce :: Position -> Position -> Position
    wallBounce pos (vx, vy) | wallCollision pos      = ( 0, 0 )
                            | wallCollision1 pos     = ( 0, 0 )
                            | wallCollision2 pos     = ( 0, 0 )
                            | wallCollision3 pos     = ( 0, 0 )
                            | wallCollision4 pos     = ( 0, 0 )
                            | wallCollision5 pos     = ( 0, 0 )
                            | wallCollision6 pos     = ( 0, 0 )
                            | wallCollision7 pos     = ( 0, 0 )
                            | wallCollision8 pos     = ( 0, 0 )
                            | wallCollision9 pos     = ( 0, 0 )
                            | wallCollision10 pos    = ( 0, 0 )
                            | wallCollision11 pos    = ( 0, 0 )
                            | upCorner || downCorner = (  0, 0 )
                            | sideCollision          = (  0, vy)
                            | topCollision pos       = ( vx, 0 )
                            | downCollision pos      = ( vx, 0 )
                            | otherwise              = ( vx, vy)
        where sideCollision = (leftCollision pos || rightCollision pos) 
                                && (not (topCollision pos) || not (downCollision pos))
              upCorner      = (leftCollision pos || rightCollision pos) && (topCollision pos)
              downCorner    = (leftCollision pos || rightCollision pos) && (downCollision pos)

    -- | verifica colisão do fantasma com o pacman
    gameOver :: Position -> Position -> Bool
    gameOver pacman ghost = distanciaEuclidiana pacman ghost <= pacmanRadius + ghostRadius

    -- | calcula a distância euclidiana
    distanciaEuclidiana :: Position -> Position -> Float
    distanciaEuclidiana (px, py) (gx, gy) = sqrt ((px-gx)^2 + (py-gy)^2)

    -- | verifica se o pacman pegou a moeda
    coletouCoin :: Position -> Coins -> Bool
    coletouCoin pacman [] = False
    coletouCoin pacman ((Coin cx cy):cs) = distanciaEuclidiana pacman (cx, cy) <= pacmanRadius + coinSize || coletouCoin pacman cs

    -- | remove a moeda que o pacman coletou
    removeCoin :: Position -> Coins -> Coins
    removeCoin pacman coins = [(Coin x y) | (Coin x y) <- coins, distanciaEuclidiana pacman (x, y) >= pacmanRadius + coinSize]